﻿// Climate Framework for Uncertainty, Negotiation and Distribution
// Version 3.5
// (c) David Anthoff and Richard S.J. Tol
// http://www.fund-model.org
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;

namespace Esmf
{
    /// <summary>
    /// Interaction logic for ModelFieldsWindow.xaml
    /// </summary>
    public partial class ModelFieldsWindow : Window
    {
        private List<ModelFields.Field> _yearFields;

        public ModelFieldsWindow(ModelFields fields)
        {
            InitializeComponent();

            var d = from p in fields.GetDimensionalFieldsOperator()
                    where (p.Values is IParameter1Dimensional<Timestep, double>) || (p.Values is IParameter2Dimensional<Timestep, Fund.CommonDimensions.Region, double>)
                    orderby p.ComponentName, p.FieldName
                    select p;

            _yearFields = new List<ModelFields.Field>(d);

            foreach (var s in d)
                yearFieldsListBox.Items.Add(string.Format("{0}.{1}", s.ComponentName, s.FieldName ));
        }

        private void yearFieldsListBox_MouseDoubleClick(object sender, System.Windows.Input.MouseButtonEventArgs e)
        {
            if (yearFieldsListBox.SelectedIndex>=0)
            {
                var selected = _yearFields[yearFieldsListBox.SelectedIndex];
                if (selected.Values is IParameter1Dimensional<Timestep, double>)
                {
                    var w = new YearFieldWindow(String.Format("{0}.{1}", selected.ComponentName, selected.FieldName), (IParameter1Dimensional<Timestep, double>)selected.Values);

                    w.Show();
                }
                else if (selected.Values is IParameter2Dimensional<Timestep, Fund.CommonDimensions.Region, double>)
                {
                    var w = new YearRegionFieldWindow(String.Format("{0}.{1}", selected.ComponentName, selected.FieldName), (IParameter2Dimensional<Timestep, Fund.CommonDimensions.Region, double>)selected.Values);

                    w.Show();
                }
            }
        }
    }
}
