﻿// Climate Framework for Uncertainty, Negotiation and Distribution
// Version 3.5
// (c) David Anthoff and Richard S.J. Tol
// http://www.fund-model.org
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Globalization;

namespace Fund
{
    public class ExponentialDistributionParameter : Parameter
    {
        private double m_value;
        private double m_lambda;
        private bool m_useLowerBound, m_useUpperBound;
        private double m_lowerBound, m_upperBound;

        protected override double GetValue()
        {
            return m_value;
        }

        public ExponentialDistributionParameter(ParameterKey Key, string[] Arguments, Random rand)
            : base(Key, rand)
        {

            m_lambda = Convert.ToDouble(Arguments[0], CultureInfo.InvariantCulture);
            m_useLowerBound = false;
            m_useUpperBound = false;

            if (Arguments.Length > 1)
            {
                if (Arguments[1].Trim().Length > 0)
                {
                    m_useLowerBound = true;
                    m_lowerBound = Convert.ToDouble(Arguments[1], CultureInfo.InvariantCulture);
                }

                if (Arguments.Length > 2)
                {
                    if (Arguments[2].Trim().Length > 0)
                    {
                        m_useUpperBound = true;
                        m_upperBound = Convert.ToDouble(Arguments[2], CultureInfo.InvariantCulture);
                    }
                }
            }

            SetToBestGuessValue();

        }

        public override void SetToRandomValue()
        {
            double i_helper;

            LegacyFund28Math.GenExponential(m_lambda, out i_helper, _rand);
            m_value = i_helper;

            while ((m_useLowerBound && (m_value < m_lowerBound)) || (m_useUpperBound && (m_value > m_upperBound)))
            {
                LegacyFund28Math.GenExponential(m_lambda, out i_helper, _rand);
                m_value = i_helper;
            }
        }

        public override void SetToBestGuessValue()
        {
            m_value = 0.0;
        }

        public double? LowerBound
        {
            get
            {
                if (m_useLowerBound)
                    return m_lowerBound;
                else
                    return null;
            }
        }

        public double? UpperBound
        {
            get
            {
                if (m_useUpperBound)
                    return m_upperBound;
                else
                    return null;
            }
        }

        public override double Mean { get { return 1.0/m_lambda; } }
        public override double StdDev { get { return 1.0/m_lambda; } }


    }
}
