﻿// Climate Framework for Uncertainty, Negotiation and Distribution
// Version 3.5
// (c) David Anthoff and Richard S.J. Tol
// http://www.fund-model.org
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Globalization;

namespace Fund
{
    public class GammaDistributionParameter : Parameter
    {
        private double m_value;
        private double m_alpha, m_beta;
        private double? m_lowerBound = null;
        private double? m_upperBound = null;

        protected override double GetValue()
        {
            return m_value;
        }

        public GammaDistributionParameter(ParameterKey Key, string[] Arguments, Random rand)
            : base(Key, rand)
        {

            m_alpha = Convert.ToDouble(Arguments[0], CultureInfo.InvariantCulture);
            m_beta = Convert.ToDouble(Arguments[1], CultureInfo.InvariantCulture);

            if (Arguments.Length > 2)
            {
                if (Arguments[2].Trim().Length > 0)
                {
                    m_lowerBound = Convert.ToDouble(Arguments[2], CultureInfo.InvariantCulture);
                }

                if (Arguments.Length > 3)
                {
                    if (Arguments[3].Trim().Length > 0)
                    {
                        m_upperBound = Convert.ToDouble(Arguments[3], CultureInfo.InvariantCulture);
                    }
                }
            }

            SetToBestGuessValue();

        }

        public override void SetToBestGuessValue()
        {
            m_value = m_beta * (m_alpha - 1.0);
        }

        public override void SetToRandomValue()
        {
            double i_helper;

            LegacyFund28Math.GenGamma(m_alpha, m_beta, out i_helper, _rand);
            m_value = i_helper;

            while ((m_lowerBound.HasValue && (m_value < m_lowerBound.Value)) ||
                (m_upperBound.HasValue && (m_value > m_upperBound.Value)))
            {
                LegacyFund28Math.GenGamma(m_alpha, m_beta, out i_helper, _rand);
                m_value = i_helper;
            }
        }

        public double? LowerBound
        {
            get
            {
                return m_lowerBound;
            }
        }

        public double? UpperBound
        {
            get
            {
                return m_upperBound;
            }
        }

        public override double Mean { get { return m_alpha*m_beta; } }
        public override double StdDev { get { return Math.Sqrt(m_alpha*m_beta*m_beta); } }

        public double Alpha { get { return m_alpha; } }

    }
}
