﻿// Climate Framework for Uncertainty, Negotiation and Distribution
// Version 3.5
// (c) David Anthoff and Richard S.J. Tol
// http://www.fund-model.org
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Fund
{
    public class RegionParameterKey : ParameterKey
    {
        private int _regionId;
        private string _region = String.Empty;

        public RegionParameterKey(string name, string region)
            : base(name)
        {
            _region = region;
            _regionId = LegacyRegionNames.GetIdFromName(region);
        }

        public RegionParameterKey(string name, int regionId)
            : base(name)
        {
            _regionId = regionId;
        }

        public override bool Equals(object obj)
        {
            if (obj == null) return false;
            if (Object.ReferenceEquals(this, obj)) return true;
            if (this.GetType() != obj.GetType()) return false;

            RegionParameterKey castedObj = (RegionParameterKey)obj;

            if (this._name.Equals(castedObj._name) && this._regionId.Equals(castedObj._regionId)) return true;

            return false;
        }

        public override int GetHashCode()
        {
            return _name.GetHashCode() ^ (_regionId<<16);
        }

        public override string ToString()
        {
            return String.Format("{0};{1}", Name, RegionName);
        }


        public string RegionName 
        {
            get
            {
                if (_region == String.Empty)
                    _region = LegacyRegionNames.GetName(_regionId);

                return _region; 
            }
        }
        public int RegionId { get { return _regionId; } }
    }
}
