﻿// Climate Framework for Uncertainty, Negotiation and Distribution
// Version 3.5
// (c) David Anthoff and Richard S.J. Tol
// http://www.fund-model.org
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Globalization;

namespace Fund
{
    public class TriangularDistributionParameter : Parameter
    {
        private double m_value;
        private double m_bestGuess;
        private double m_min, m_max;

        protected override double GetValue()
        {
            return m_value;
        }

        public TriangularDistributionParameter(ParameterKey Key, string[] Arguments, Random rand)
            : base(Key, rand)
        {
            m_min = Convert.ToDouble(Arguments[0], CultureInfo.InvariantCulture);
            m_bestGuess = Convert.ToDouble(Arguments[1], CultureInfo.InvariantCulture);
            m_max = Convert.ToDouble(Arguments[2], CultureInfo.InvariantCulture);
            SetToBestGuessValue();

        }

        public override void SetToRandomValue()
        {
            double i_helper;

            LegacyFund28Math.GenTriang(m_min, m_max, m_bestGuess, out i_helper, _rand);
            m_value = i_helper;
        }

        public override void SetToBestGuessValue()
        {
            m_value = m_bestGuess;
        }

        public override double Mean { get { return (m_bestGuess + m_min + m_max) / 3.0; } }
        public override double StdDev { get { return Math.Sqrt((m_bestGuess * m_bestGuess + m_min * m_min + m_max * m_max - m_bestGuess * m_min - m_bestGuess * m_max - m_min * m_max) / 18.0); } }

    }
}
