﻿// Climate Framework for Uncertainty, Negotiation and Distribution
// Version 3.5
// (c) David Anthoff and Richard S.J. Tol
// http://www.fund-model.org
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Diagnostics;

namespace Esmf
{
    public class Variable2Dimensional<D1,D2,T> : IVariable2Dimensional<D1, D2, T>, IParameter2Dimensional<D1, D2, T>
    {
        private T[,] _values;
        private Enum2IntConversionDelegate<D1> _enumConversionDelegate1;
        private Enum2IntConversionDelegate<D2> _enumConversionDelegate2;
        private ModelFields _parent;

        public Variable2Dimensional(ModelFields parent)
        {
            // TODO Change this to properly investigate the dimensions
            _values = new T[16, 16];

            _enumConversionDelegate1 = Enum2IntConversionSingleton<D1>.Instance.Delegate;
            _enumConversionDelegate2 = Enum2IntConversionSingleton<D2>.Instance.Delegate;

            _parent = parent;
        }

        [Conditional("FUNDNANCHECK")]
        private void CheckForValidValue(T v)
        {
            if (v.GetType() == typeof(double))
            {
                double d = ((double)((object)v));

                if (double.IsNaN(d))
                    throw new ArgumentOutOfRangeException("NaN is not allowed as a value");
                else if (double.IsInfinity(d))
                    throw new ArgumentOutOfRangeException("Infinity is not allowed as a value");
            }
        }

        public T this[D1 index1, D2 index2]
        {
            get
            {
                return _values[_enumConversionDelegate1(index1) - 1, _enumConversionDelegate2(index2) - 1];
            }
            set
            {
                CheckForValidValue(value);

                _values[_enumConversionDelegate1(index1) - 1, _enumConversionDelegate2(index2) - 1] = value;
            }
        }


        #region IVariable2Dimensional<D1,D2,T> Members

        T IVariable2Dimensional<D1, D2, T>.this[D1 index1, D2 index2]
        {
            get
            {
                return this[index1, index2];
            }
            set
            {
                this[index1, index2] = value;
            }
        }

        IEnumerable<Parameter2DimensionalMember<D1, D2, T>> IVariable2Dimensional<D1, D2, T>.GetEnumerator()
        {
            return (this as IParameter2Dimensional<D1, D2, T>).GetEnumerator();
        }

        #endregion

        #region IParameter2Dimensional<D1,D2,T> Members

        T IParameter2Dimensional<D1, D2, T>.this[D1 index1, D2 index2]
        {
            get
            {
                return this[index1, index2];
            }
        }

        IEnumerable<Parameter2DimensionalMember<D1, D2, T>> IParameter2Dimensional<D1, D2, T>.GetEnumerator()
        {
            D1[] values1 = (D1[])Enum.GetValues(typeof(D1));
            D2[] values2 = (D2[])Enum.GetValues(typeof(D2));
            for (int i = 0; i < 16; i++)
            {
                for (int l = 0; l < 16; l++)
                {
                    yield return new Parameter2DimensionalMember<D1,D2,T>(values1[i], values2[l],_values[i,l], _parent.AttachedDimensions);
                }
            }
        }

        #endregion

    }
}
