// Climate Framework for Uncertainty, Negotiation and Distribution
// Version 3.5
// (c) David Anthoff and Richard S.J. Tol
// http://www.fund-model.org
using System;
using System.Collections.Generic;
using System.Text;
using System.Diagnostics;

namespace Esmf
{
    public class VariableWithTimeDimension<T> : IVariable1Dimensional<Timestep, T>, IParameter1Dimensional<Timestep, T>
    {
        private T[] _values;
        private ModelFields _parent;
        private const int _NYears = 1050;

        public VariableWithTimeDimension(ModelFields parent)
        {
            // TODO Change this to properly find out how many years the model run will have
            _values = new T[_NYears];

            _parent = parent;
        }

        [Conditional("FUNDNANCHECK")]
        private void CheckForValidValue(T v)
        {
            if (v.GetType() == typeof(double))
            {
                double d = ((double)((object)v));

                if (double.IsNaN(d))
                    throw new ArgumentOutOfRangeException("NaN is not allowed as a value");
                else if (double.IsInfinity(d))
                    throw new ArgumentOutOfRangeException("Infinity is not allowed as a value");
            }
        }

        public T this[Timestep index]
        {
            get
            {
                return _values[index.Value];
            }
            set
            {
                CheckForValidValue(value);
                _values[index.Value] = value;
            }
        }

        #region IVariable1Dimensional<Timestep,T> Members

        T IVariable1Dimensional<Timestep, T>.this[Timestep index]
        {
            get
            {
                return this[index];
            }
            set
            {
                this[index] = value;
            }
        }

        IEnumerable<Parameter1DimensionalMember<Timestep, T>> IVariable1Dimensional<Timestep, T>.GetEnumerator()
        {
            return (this as IParameter1Dimensional<Timestep, T>).GetEnumerator();
        }

        #endregion

        #region IParameter1Dimensional<Timestep,T> Members

        T IParameter1Dimensional<Timestep, T>.this[Timestep index]
        {
            get
            {
                return this[index];
            }
        }

        IEnumerable<Parameter1DimensionalMember<Timestep, T>> IParameter1Dimensional<Timestep, T>.GetEnumerator()
        {
            for (int i = 0; i < _NYears; i++)
            {
                yield return new Parameter1DimensionalMember<Timestep, T>(Timestep.FromSimulationYear(i), _values[i], _parent.AttachedDimensions);
            }
        }

        #endregion

    }
}
