﻿// Climate Framework for Uncertainty, Negotiation and Distribution
// Version 3.5
// (c) David Anthoff and Richard S.J. Tol
// http://www.fund-model.org
using Esmf;
using System;

namespace Fund
{

    public interface IFundOld2Climate
    {
        IVariable1Dimensional<Timestep, double> addco2 { get; }
        IVariable1Dimensional<Timestep, double> addch4 { get; }
        IVariable1Dimensional<Timestep, double> addn2o { get; }
        IVariable1Dimensional<Timestep, double> addsf6 { get; }
    }

    public class FundOld2ClimateComponent
    {
        public void Run(Clock clock, IFundOld2Climate state, FundParameter parameters)
        {
            var t = clock.Current;

            state.addch4[t] = 0;
            state.addco2[t] = 0;
            state.addn2o[t] = 0;
            state.addsf6[t] = 0;

            if ((t.Value >= (parameters.emissionPeriod - 1950)) && (t.Value < (parameters.emissionPeriod - 1950 + 10)))
            {
                switch ((int)parameters.marginalGas)
                {
                    case 0: state.addco2[t] = 1; break;
                    case 1: state.addch4[t] = 1; break;
                    case 2: state.addn2o[t] = 1; break;
                    case 3: state.addsf6[t] = 1; break;
                    default: throw new Exception();
                }
            }

        }
    }

}