﻿// Climate Framework for Uncertainty, Negotiation and Distribution
// Version 3.5
// (c) David Anthoff and Richard S.J. Tol
// http://www.fund-model.org
using System.Collections.Generic;
using System.Reflection.Emit;
using Esmf;
using System;

namespace Fund
{

    public class RegionRegionParameterConnector<D1, D2> : Esmf.IParameter2Dimensional<D1, D2, double>
    {
        private double[,] _values = new double[16, 16];
        private Enum2IntConversionDelegate<D1> _enumConversionDelegate1;
        private Enum2IntConversionDelegate<D2> _enumConversionDelegate2;

        public double this[D1 d1, D2 d2] { get { return GetItem(d1, d2); } }

        public double GetItem(D1 d1, D2 d2)
        {
            return _values[_enumConversionDelegate1(d1) - 1, _enumConversionDelegate2(d2) - 1];
        }

        public RegionRegionParameterConnector(Parameters param, string name)
        {
            if ((typeof(D1).BaseType == typeof(System.Enum)) && (typeof(D2).BaseType == typeof(System.Enum)))
            {
                var values1 = (int[])Enum.GetValues(typeof(D1));
                var values2 = (int[])Enum.GetValues(typeof(D2));

                foreach (int r in values1)
                {
                    foreach (int rr in values2)
                    {
                        _values[r - 1, rr - 1] = param.GetItemByKey(new RegionRegionParameterKey(name, Enum.GetName(typeof(D1), r), Enum.GetName(typeof(D2), rr)));
                    }
                }

                _enumConversionDelegate1 = Enum2IntConversionSingleton<D1>.Instance.Delegate;
                _enumConversionDelegate2 = Enum2IntConversionSingleton<D2>.Instance.Delegate;
            }
            else
                throw new InvalidOperationException();

        }

        public System.Collections.Generic.IEnumerable<Esmf.Parameter2DimensionalMember<D1, D2, System.Double>> GetEnumerator()
        {
            throw new NotImplementedException();
        }

    }
}