﻿// Climate Framework for Uncertainty, Negotiation and Distribution
// Version 3.5
// (c) David Anthoff and Richard S.J. Tol
// http://www.fund-model.org
using Esmf;
using System.Collections.Generic;
using System.Reflection.Emit;
using System;

namespace Fund
{

    public class YearRegionParameterConnector<D2> : Esmf.IParameter2Dimensional<Esmf.Timestep, D2, double>
    {
        private double[,] _values = new double[LegacyConstants.NYear+1, 16];
        private Enum2IntConversionDelegate<D2> _enumConversionDelegate;

        public double this[Esmf.Timestep t, D2 d] { get { return GetItem(t, d); } }

        public double GetItem(Esmf.Timestep t, D2 d)
        {
            return _values[t.Value, _enumConversionDelegate(d) - 1];
        }

        public YearRegionParameterConnector(Parameters param, string name)
        {
            for (int i = 0; i <= LegacyConstants.NYear; i++)
            {
                foreach (int r in ((int[])Enum.GetValues(typeof(D2))))
                    _values[i, r - 1] = param.GetItemByKey(new YearRegionParameterKey(name, i, Enum.GetName(typeof(D2), r)));

                _enumConversionDelegate = Enum2IntConversionSingleton<D2>.Instance.Delegate;

            }
        }

        public System.Collections.Generic.IEnumerable<Esmf.Parameter2DimensionalMember<Esmf.Timestep, D2, System.Double>> GetEnumerator()
        {
            throw new NotImplementedException();
        }
    }
}