﻿// Climate Framework for Uncertainty, Negotiation and Distribution
// Version 3.5
// (c) David Anthoff and Richard S.J. Tol
// http://www.fund-model.org
namespace Fund
{

    // This class implements an equity weighting scheme based on average income:
    // equity weight = average world per capita income / average regional per capiat income
    public class AverageEquityWeighting : WeightingScheme
    {

        public override void CalculateWeights(ModelOutput ModelOutput)
        {
            double i_worldIncome, i_worldPopulation;
            double i_equityWeight;

            // TODO -oDavid Anthoff: Change start of "real" period to constant
            for (int i = 50; i <= LegacyConstants.NYear; i++)
            {
                i_worldIncome = 0;
                i_worldPopulation = 0;

                for (int l = 1; l <= LegacyConstants.NoReg; l++)
                {
                    i_worldIncome = i_worldIncome + ModelOutput.Incomes[i, l];
                    i_worldPopulation = i_worldPopulation + ModelOutput.Populations[i, l];
                }

                for (int l = 1; l <= LegacyConstants.NoReg; l++)
                {
                    i_equityWeight = (i_worldIncome / i_worldPopulation) / (ModelOutput.Incomes[i, l] / ModelOutput.Populations[i, l]);
                    AddWeight(i, l, i_equityWeight);
                }
            }
        }

        protected override string GetName()
        {
            return "average equity weighting";
        }
    }
}