﻿// Climate Framework for Uncertainty, Negotiation and Distribution
// Version 3.5
// (c) David Anthoff and Richard S.J. Tol
// http://www.fund-model.org
using System;

namespace Fund
{
    public class HeterogeneousPRTP : WeightingScheme
    {
        private double m_Mju;

        public override void CalculateWeights(ModelOutput ModelOutput)
        {
            double i_prtp;
            double i_newPerCapitaIncome, i_oldPerCapitaIncome;
            double i_perCapitaGrowthRate;
            double i_discountFactor;

            for (int i = 1; i <= LegacyConstants.NoReg; i++)
            {
                i_discountFactor = 1;
                // TODO -oDavid Anthoff: Change start of "real" period to constant
                for (int l = 50; l <= LegacyConstants.NYear; l++)
                {
                    AddWeight(l, i, i_discountFactor);
                    i_newPerCapitaIncome = ModelOutput.Incomes[l, i] / ModelOutput.Populations[l, i];
                    i_oldPerCapitaIncome = ModelOutput.Incomes[l - 1, i] / ModelOutput.Populations[l - 1, i];
                    i_perCapitaGrowthRate = (i_newPerCapitaIncome - i_oldPerCapitaIncome) / i_oldPerCapitaIncome;

                    i_prtp = 2.0 / ((l - 50.0) + 1.0 / m_Mju);

                    i_discountFactor = i_discountFactor / (1.00 + i_prtp + i_perCapitaGrowthRate);
                }
            }
        }

        public HeterogeneousPRTP(double Mju)
            : base()
        {
            m_Mju = Mju;
        }

        protected override string GetName()
        {
            return "Heterogeneous prtp (mju=" + Convert.ToString(m_Mju * 100) + "%)";
        }

    }
}